/**
 * JHylaFax - A java client for HylaFAX.
 *
 * Copyright (C) 2005 by Steffen Pingel <steffenp@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package net.sf.jhylafax;

import gnu.hylafax.Job;
import gnu.inet.ftp.ServerResponseException;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.TimeZone;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Provides static methods to handle the server communication.
 * 
 * @author Steffen Pingel
 */
public class HylaFAXClientHelper extends Thread {
	
	private final static Log logger = LogFactory.getLog(HylaFAXClientHelper.class);
	
	final static String QUEUE_SEPARATOR = "|";
	
	// number | priority | id | length | notification | owner | res | width | tts 
	// tag | pages | dials | max dials | tries | max tries | status
	final static String JOBFMT = "__JOBFMT |%e |%i |%j |%l |%n |%o |%r |%w |%z |%J |%P |%d |%x |%t |%u |%s ";
	
	// error | filename | rights | size | owner | pages | sender | time | recveiving
	final static String RCVFMT = "__RCVFMT |%e |%f |%m |%n |%o |%p |%s |%t |%z ";
	
	// 
	final static String FILEFMT = "__FILEFMT |%f |%o |%q |%s |%u |%a ";
	
	public static String calculateTime(Date sendTime, String timeZoneID) {
		String time = "NOW";
		
		if (sendTime != null) {
			int offset = 0;
			long date = sendTime.getTime();
			
			TimeZone tz = TimeZone.getTimeZone(timeZoneID);
			
//			stz.setStartRule(Calendar.MARCH, -1, Calendar.SUNDAY,  2*60*60*1000);
//			stz.setEndRule(Calendar.OCTOBER, -1, Calendar.SUNDAY,  2*60*60*1000);
			
			if (tz.inDaylightTime(sendTime)) {
				offset = 3600 * 1000;
			}
			
			date -= offset + tz.getRawOffset();
			sendTime = new Date(date);
			
			time = (new java.text.SimpleDateFormat("yyyyMMddHHmm")).format(sendTime);
		}
		
		return time;
	}
	
	public static String convertMonth(String date) {
		String m = "";
		
		if (date.regionMatches(2, "Jan", 0, 3)) {
			m = "01";
		} else if (date.regionMatches(2, "Feb", 0, 3)) {
			m = "02";
		} else if (date.regionMatches(2, "Mar", 0, 3)) {
			m = "03";
		} else if (date.regionMatches(2, "Apr", 0, 3)) {
			m = "04";
		} else if (date.regionMatches(2, "Mai", 0, 3)) {
			m = "05";
		} else if (date.regionMatches(2, "Jun", 0, 3)) {
			m = "06";
		} else if (date.regionMatches(2, "Jul", 0, 3)) {
			m = "07";
		} else if (date.regionMatches(2, "Aug", 0, 3)) {
			m = "08";
		} else if (date.regionMatches(2, "Sep", 0, 3)) {
			m = "09";
		} else if (date.regionMatches(2, "Oct", 0, 3)) {
			m = "10";
		} else if (date.regionMatches(2, "Nov", 0, 3)) {
			m = "11";
		} else if (date.regionMatches(2, "Dec", 0, 3)) {
			m = "12";
		}
		
		if (m.equals("")) {
			// could not replace
			return date;
		} else {
			// return text with number
			return date.substring(0, 2) + m + date.substring(5);
		}
	}		
	public static Date parseDate(String date, boolean future) throws ParseException {
		if (date.length() == 5) {
			// time i.e. 15:00
			return (new java.text.SimpleDateFormat("hh:mm")).parse(date);
		} else
			if (date.length() == 7) {
				// date i.e. 03Jan00
				String m = parseMonthToNumber(date.substring(2, 5));
				
				if (m != null) {
					// replace text with number
					m = date.substring(0, 2) + m + date.substring(5);
					return (new SimpleDateFormat("ddMMyy")).parse(m);
				} else {
					// date (this week) Wed07PM
					return parseWeekDate(date, future);
				}
			}
		
		return null;
	}
	
	public final static HylaFAXJob parseFmt(String response) {
		logger.debug("Received: " + response);
		
		if (response.trim().length() == 0) {
			// work around a bug in HylaFax
			return null;
		}
		if (response.startsWith("__JOBFMT")) {
			return parseJobFmt(response);
		}
		else if (response.startsWith("__RCVFMT")) {
			return parseRcvFmt(response);
		}
		else if (response.startsWith("__FILEFMT")) {
			return parseFileFmt(response);
		}
		else {
			logger.error("Invalid response: " + response);
			return null;
		}
	}

	public final static HylaFAXJob parseFileFmt(String response) {
		StringTokenizer st = new StringTokenizer(response, QUEUE_SEPARATOR);
		StringTokenizer jf = new StringTokenizer(FILEFMT, QUEUE_SEPARATOR);
		HylaFAXJob job = new HylaFAXJob();
		while (st.hasMoreElements() && jf.hasMoreElements()) {
			char c = jf.nextToken().charAt(1);
			String s = st.nextToken().trim();
			if (s.length() > 0) {
				try {
					// parse
					switch (c) {
					case 'a': // Last access time
						break;
					case 'c': // Creation time
						job.setSendTime(parseDate(s, false));
						break;
					case 'd': // Device number (octal)
						break;
					case 'f':
						job.setFilename(s);
						break;
					case 'g': // Group identifier (decimal)
						break;
					case 'i': // Inode number (decimal)
						break;
					case 'l': // Link count (decimal)
						break;
					case 'm': // Last modification time
						break;
					case 'o':
						job.setOwner(s);
						break;
					case 'p': // Fax-style protection flags (no group bits)
						break;
					case 'q':
						job.setPermissions(s);
						break;
					case 'r': // Root device number (octal)
						break;
					case 's':
						job.setFilesize(Long.parseLong(s));
						break;
					case 'u': // User identifier (decimal)
						break;
					}
				}
				catch (NumberFormatException e) {
					logger.info("Error parsing respone", e);
				}
				catch (ParseException e) {
					logger.info("Error parsing respone", e);
				}
			}
		}
		return job;
	}
				
	public final static HylaFAXJob parseJobFmt(String response) {
		StringTokenizer st = new StringTokenizer(response, QUEUE_SEPARATOR);
		StringTokenizer jf = new StringTokenizer(JOBFMT, QUEUE_SEPARATOR);
		HylaFAXJob job = new HylaFAXJob();
		while (st.hasMoreElements() && jf.hasMoreElements()) {
			char c = jf.nextToken().charAt(1);
			String s = st.nextToken().trim();
			if (s.length() > 0) {
				// parse
				switch (c) {
				case 'd' :
					try {
						job.setDialsAttempted((new Integer(s)).intValue());
					} catch (NumberFormatException e) {
					}
					break;
				case 'e' :
					job.setNumber(s);
					break;
				case 'i' :
					try {
						job.setPriority((new Integer(s)).intValue());
					} catch (NumberFormatException e) {
					}
					break;
				case 'j' :
					try {
						job.setID((new Integer(s)).intValue());
					} catch (NumberFormatException e) {
					}
					break;
				case 'l' :
					try {
						job.setPaperHeight(Integer.parseInt(s));
					} catch (NumberFormatException e) {
					}
					break;
				case 'n' :
					switch (s.charAt(0)) {
					case 'D' :
						job.setNotify(Job.NOTIFY_DONE);
						break;
					case 'Q' :
						job.setNotify(Job.NOTIFY_REQUEUE);
						break;
					case 'A' :
						job.setNotify(Job.NOTIFY_ALL);
						break;
					default :
						job.setNotify(Job.NOTIFY_NONE);
					}
					break;
				case 'o' :
					job.setOwner(s);
					break;
				case 'r' :
					try {
						job.setResolution((new Integer(s)).intValue());
					} catch (NumberFormatException e) {
					}
					break;
				case 's' :
					job.setLastError(s);
					break;
				case 't' :
					try {
						job.setTriesAttempted((new Integer(s)).intValue());
					} catch (NumberFormatException e) {
					}
					break;
				case 'u' :
					try {
						job.setMaxTries((new Integer(s)).intValue());
					} catch (NumberFormatException e) {
					}
					break;
				case 'x' :
					try {
						job.setMaxDials((new Integer(s)).intValue());
					} catch (NumberFormatException e) {
					}
					break;
				case 'w' :
					try {
						job.setPaperWidth(Integer.parseInt(s));
					} catch (NumberFormatException e) {
					}
					break;
				case 'z' :
					try {
						Date date = parseDate(s, true);
						job.setSendTime(date);
					} catch (Exception e) {
					}
					break;
				case 'D' :
					try {
						StringTokenizer t = new StringTokenizer(s, ":");
						if (st.hasMoreTokens())
							job.setDialsAttempted((new Integer(t.nextToken().trim())).intValue());
						if (st.hasMoreTokens())
							job.setMaxDials((new Integer(t.nextToken().trim())).intValue());
					} catch (NumberFormatException e) {
					}
					break;
				case 'P' :
					try {
						StringTokenizer t = new StringTokenizer(s, ":");
						if (st.hasMoreTokens())
							job.setPagesTransmitted((new Integer(t.nextToken().trim())).intValue());
						if (st.hasMoreTokens())
							job.setPageCount((new Integer(t.nextToken().trim())).intValue());
					} catch (NumberFormatException e) {
					}
					break;
				}
			}
		}
		return job;
	}

	public static String parseMonthToNumber(String month)
	{
		if (month.equals("Jan")) {
			return "01";
		}
		else if (month.equals("Feb")) {
			return "02";
		}
		else if (month.equals("Mar")) {
			return "03";
		}
		else if (month.equals("Apr")) {
			return "04";
		}
		else if (month.equals("May")) {
			return "05";
		}
		else if (month.equals("Jun")) {
			return "06";
		}
		else if (month.equals("Jul")) {
			return "07";
		}
		else if (month.equals("Aug")) {
			return "08";
		}
		else if (month.equals("Sep")) {
			return "09";
		}
		else if (month.equals("Oct")) {
			return "10";
		}
		else if (month.equals("Nov")) {
			return "11";
		}
		else if (month.equals("Dec")) {
			return "12";
		}
		return null;
	}
	
	public final static HylaFAXJob parseRcvFmt(String response) {
		StringTokenizer st = new StringTokenizer(response, QUEUE_SEPARATOR);
		StringTokenizer jf = new StringTokenizer(RCVFMT, QUEUE_SEPARATOR);
		HylaFAXJob job = new HylaFAXJob();
		while (st.hasMoreElements() && jf.hasMoreElements()) {
			char c = jf.nextToken().charAt(1);
			String s = st.nextToken().trim();
			if (s.length() > 0) {
				try {
					switch (c) {
					case 'e' :
						job.setLastError(s);
						break;
					case 'f' :
						job.setFilename(s);
						break;
					case 'm' :
						job.setPermissions(s);
						break;
					case 'n' :
						job.setFilesize(Long.parseLong(s));
						break;
					case 'o' :
						job.setOwner(s);
						break;
					case 'p' :
						job.setPageCount(Integer.parseInt(s));
						break;
					case 's' :
						job.setSender(s);
						break;
					case 't' :
						job.setSendTime(parseDate(s, false));
						break;
					case 'z' :
						job.setReceiving(s.equals("*"));
						break;
					}
				} 
				catch (ParseException e) {
					logger.info("Error parsing respone", e);
				}
				catch (NumberFormatException e) {
					logger.info("Error parsing respone", e);
				}
			}
		}
		return job;
	}
	
	public static Date parseWeekDate(String date, boolean future) {
		String str = date.substring(0, 3);
		Calendar c = Calendar.getInstance();
		int day = 0;
		int hour = 0;
		long diff = 0;
		
		if (str.equals("Sun")) {
			day = 1;		
		} else if (str.equals("Mon")) {
			day = 2;
		} else if (str.equals("Tue")) {
			day = 3;
		} else if (str.equals("Wed")) {
			day = 4;
		} else if (str.equals("Thu")) {
			day = 5;
		} else if (str.equals("Fri")) {
			day = 6;
		} else if (str.equals("Sat")) {
			day = 7;
		} else {
			return null;
		}
		
		if (future && (day <= c.get(Calendar.DAY_OF_WEEK))) day += 7;
		
		diff = (day - c.get(Calendar.DAY_OF_WEEK)) * 86400000;
		
		// hour
		try {
			hour = (new Integer(date.substring(3, 5))).intValue();
		} catch (NumberFormatException e) {
			return null;
		}
		if (date.substring(5).equals("PM")) {
			hour += 12;
		}
		c.set(c.get(Calendar.YEAR), c.get(Calendar.MONTH), c.get(Calendar.DAY_OF_MONTH), hour, 0);
		
		return (new Date(c.getTime().getTime() + diff));
	}

	public final static void initializeFromSettings(HylaFAXJob job) {
		job.setSender(Settings.FULLNAME.getValue());
		job.setNotifyAdress(Settings.EMAIL.getValue());
		job.setMaxDials(Settings.MAXDIALS.getValue());
		job.setMaxTries(Settings.MAXTRIES.getValue());
		job.setNotify(Settings.NOTIFICATION.getValue().getCommand());
		job.setPaperHeight(Settings.PAPER.getValue().getHeight());
		job.setPaperWidth(Settings.PAPER.getValue().getWidth());
		job.setPriority(Settings.PRIORITY.getValue());
		job.setResolution(Settings.RESOLUTION.getValue().getLinesPerInch());
	}

	public static void applyParameter(Job faxJob, HylaFAXJob job) throws ServerResponseException, IOException
	{
		//faxJob.setChopThreshold(3);
		faxJob.setDialstring(job.getNumber());
		if (job.getSender() != null && job.getSender().trim().length() > 0) {
			faxJob.setFromUser(job.getSender());
		}
		faxJob.setKilltime("000259");
		faxJob.setMaximumDials(job.getMaxDials());
		faxJob.setMaximumTries(job.getMaxDials());
		if (job.getNotifyAdress() != null && job.getNotifyAdress().trim().length() > 0) {
			faxJob.setNotifyAddress(job.getNotifyAdress());
		}
		faxJob.setNotifyType(job.getNotify());
		faxJob.setPageChop(Job.CHOP_DEFAULT);
		faxJob.setPageWidth(job.getPaperWidth());
		faxJob.setPageLength(job.getPaperHeight());
		faxJob.setPriority(job.getPriority());
		faxJob.setProperty("SENDTIME", calculateTime(job.getSendTime(), Settings.TIMEZONE.getValue()));
		faxJob.setVerticalResolution(job.getResolution());
	}
	
}