/**
 * JHylaFax - A java client for HylaFAX.
 *
 * Copyright (C) 2005 by Steffen Pingel <steffenp@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package net.sf.jhylafax;

import gnu.hylafax.HylaFAXClient;
import gnu.inet.ftp.ConnectionEvent;
import gnu.inet.ftp.ConnectionListener;
import gnu.inet.ftp.ServerResponseException;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import javax.swing.Action;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xnap.commons.gui.AboutDialog;
import org.xnap.commons.gui.Dialogs;
import org.xnap.commons.gui.ErrorDialog;
import org.xnap.commons.gui.ProgressDialog;
import org.xnap.commons.gui.action.AbstractXNapAction;
import org.xnap.commons.gui.util.GUIHelper;
import org.xnap.commons.gui.util.IconHelper;
import org.xnap.commons.i18n.I18n;
import org.xnap.commons.io.Job;
import org.xnap.commons.io.ProgressMonitor;
import org.xnap.commons.util.FileHelper;

/**
 * The main class, that provides the main window as well as the network connection.
 * 
 * @author Steffen Pingel
 */
public class JHylaFAX extends JFrame implements LocaleChangeListener {

	public final static Locale[] SUPPORTED_LOCALES = { Locale.GERMAN, Locale.ENGLISH };
	
	private final static String[] QUEUES = { "recvq", "sendq", "pollq", "doneq", "docq" };
	
	private final static String[][] VISIBLE_COLUMNS = { 
		{ "permissions", "owner", "sender", "pages", "time", "filename" },
		{ "id", "priority", "owner", "number", "pages", "dials", "time", "error" },
		{ "id", "priority", "owner", "number", "pages", "dials", "time", "error" },
		{ "id", "priority", "owner", "number", "pages", "dials", "time", "error" },
		{ "permissions", "owner", "time", "filename" },
	};

	private final static Log logger = LogFactory.getLog(JHylaFAX.class);
	private static JHylaFAX app;
	private JTabbedPane mainTabbedPane;
	private JMenu fileMenu;
	private SettingsDialogAction settingsDialogAction;
	private JPanel statusBarPanel;
	private JLabel statusLabel;
	private JLabel serverInfoLabel;
	private ExitAction exitAction;
	private UpdateStatusAction updateStatusAction;
	private HylaFAXClient connection;
	private JToolBar mainToolBar;
	private Map<String, QueuePanel> queuePanelByName;
	private SendAction sendAction;
	private PollAction pollAction;
	private JMenu aboutMenu;
	private AboutAction aboutAction;
	private String password;
	private ResourceBundle resources;
	private List<LocaleChangeListener> localeListeners = new ArrayList<LocaleChangeListener>();
	
	public JHylaFAX() {
		app = this;
		
		try {
			Settings.load(getSettingsFile());
		}
		catch (FileNotFoundException e) {
			logger.debug("Error loading settings", e);
			// ignore, probably the first time
		}
		catch (IOException e) {
			logger.debug("Error loading settings", e);
			ErrorDialog.showError(this, I18n.tr("Could not load settings"),
					I18n.tr("JHylaFAX Error"),
					e);
		}		

        updateResourceBundle();
	
		initialize();
		Settings.MAIN_WINDOW.load(JHylaFAX.this);
		
		for (int i = 0; i < QUEUES.length; i++) {
			QueuePanel panel = queuePanelByName.get(QUEUES[i]);
			panel.restoreLayout(Settings.backstore, VISIBLE_COLUMNS[i]);
		}
		
		PropertyChangeListener connectionHandler = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if (connection != null) {
					try {
						connection.quit();
					}
					catch (Exception e) {
						logger.debug("Error closing connection", e);
					}
					connection = null;
				}
			}			
		};
		Settings.HOSTNAME.addPropertyChangeListener(connectionHandler);
		Settings.PORT.addPropertyChangeListener(connectionHandler);
		Settings.USERNAME.addPropertyChangeListener(connectionHandler);
		
		Settings.LOCALE.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				updateResourceBundle();
				
				// FIXME: recursive
				updateLabels();
			}
		});
	}
	
	private void updateResourceBundle() {
		if (resources != null) {
			I18n.removeResources(resources);
			resources = null;
		}
		try {
			resources = ResourceBundle.getBundle("i18n.jhylafax",
					Settings.LOCALE.getValue());
			I18n.addResources(resources);
        }
		catch (MissingResourceException e) {
			logger.debug("Error loading resource bundle", e);
			ErrorDialog.showError(this, I18n.tr("Could not load language support"),
					I18n.tr("JHylaFAX Error"),
					e);            
		}
		
		for (LocaleChangeListener listener : localeListeners) {
			listener.updateLabels();
		}
	}

	private void initialize() {
		initializeActions();
		initializeMenuBar();
		initializeContent();
		initializeToolBar();
		initializeStatusBar();
		
		updateLabels();		
		updateServerInfo();
	}

	private void initializeActions() {
		updateStatusAction = new UpdateStatusAction();
		settingsDialogAction = new SettingsDialogAction();
		exitAction = new ExitAction();
		sendAction = new SendAction();
		pollAction = new PollAction();
		aboutAction = new AboutAction();
	}
	
	private void initializeContent() {
		setTitle("JHylaFAX");
		setLayout(new BorderLayout());
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent event) {
				exit();
			}
		});
		
		mainTabbedPane = new JTabbedPane();
		mainTabbedPane.setBorder(GUIHelper.createEmptyBorder(10));
		getContentPane().add(mainTabbedPane, BorderLayout.CENTER);

		queuePanelByName = new HashMap<String,QueuePanel>();
		for (String name : QUEUES) {
			QueuePanel panel = new QueuePanel(name);
			queuePanelByName.put(name, panel);
			mainTabbedPane.add(panel);
		}
	}
	
	public void exit()
	{
		Settings.MAIN_WINDOW.store(JHylaFAX.this);
		for (QueuePanel panel : queuePanelByName.values()) {
			panel.saveLayout(Settings.backstore);
		}
		try {
			Settings.store(getSettingsFile());
		}
		catch (IOException e) {
			logger.debug("Error storing settings", e);
			ErrorDialog.showError(JHylaFAX.this, 
					I18n.tr("Could not store settings"), 
					I18n.tr("JHylaFAX Error"), 
					e);	
		}
		System.exit(0);
	}

	private void initializeToolBar() {
		mainToolBar = new JToolBar();
		//mainToolBar.setBorderPainted(false);
		//mainToolBar.setRollover(true);
		getContentPane().add(mainToolBar, BorderLayout.NORTH);
		
		mainToolBar.add(sendAction.createToolBarButton());
		mainToolBar.add(updateStatusAction.createToolBarButton());
		mainToolBar.addSeparator();
		mainToolBar.add(settingsDialogAction.createToolBarButton());
	}
	
	private void initializeMenuBar() {
		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		
		fileMenu = new JMenu();
		menuBar.add(fileMenu);
		fileMenu.add(sendAction.createJMenuItem());
		fileMenu.add(pollAction.createJMenuItem());
		fileMenu.add(updateStatusAction.createJMenuItem());
		fileMenu.addSeparator();
		fileMenu.add(settingsDialogAction.createJMenuItem());
		fileMenu.addSeparator();
		fileMenu.add(exitAction.createJMenuItem());
		
		aboutMenu = new JMenu();
		menuBar.add(aboutMenu);
		aboutMenu.add(aboutAction.createJMenuItem());
	}
	
	private void initializeStatusBar() {
		statusBarPanel = new JPanel(new BorderLayout());
		getContentPane().add(statusBarPanel, BorderLayout.SOUTH);
		
		statusLabel = new JLabel();
		statusBarPanel.add(statusLabel, BorderLayout.CENTER);

		serverInfoLabel = new JLabel();
		statusBarPanel.add(serverInfoLabel, BorderLayout.EAST);
	}
	
	public static JHylaFAX getInstance() {
		return app;
	}
		
	public synchronized HylaFAXClient getConnection(ProgressMonitor monitor) throws IOException, ServerResponseException {
		if (connection == null) {
			connection = new HylaFAXClient();
			logger.info("Connecting to HylaFAX server at " + Settings.HOSTNAME.getValue() + ":" + Settings.PORT.getValue());
			connection.addConnectionListener(new ConnectionListener() {
				public void connectionOpened(ConnectionEvent event) {
					logger.info("Connected to "
							+ event.getRemoteInetAddress().getHostAddress() 
							+ ":" + event.getRemotePort());
				}
				public void connectionClosed(ConnectionEvent event) {
					logger.info("Disconnected from "
							+ event.getRemoteInetAddress().getHostAddress() 
							+ ":" + event.getRemotePort());
					connection = null;
				}
				public void connectionFailed(Exception e) {
					logger.info("Connection failed", e);
					connection = null;
				}			
			});
		}
		
		connection.open(Settings.HOSTNAME.getValue(), Settings.PORT.getValue());
		while (true) {
			if (connection.user(Settings.USERNAME.getValue())) {
				password = Settings.PASSWORD.getValue();
				if (password.length() == 0) {
					password = requestPassword(monitor);
					if (password == null) {
						throw new UserAbortException();
					}
				}
				try {
					connection.pass(password);
				} catch (ServerResponseException e) {
					// wrong password, try again
					continue;
				}
			}
			break;
		}		
		return connection;
	}

	private String requestPassword(final ProgressMonitor monitor) throws ServerResponseException {
		final String[] password = new String[1];
		Runnable runner = new Runnable() {
			public void run() {
				String host = Settings.USERNAME.getValue() + "@" 
					+ Settings.HOSTNAME.getValue() + ":" 
					+ Settings.PORT.getValue();
				password[0] = Dialogs.requestPassword(
						(monitor instanceof Component) 
						? (Component)monitor
						: JHylaFAX.getInstance(), 
						I18n.tr("Enter password for {0}:", host),
						I18n.tr("JHylaFAX Connection"));
			}
		};
		try {
			SwingUtilities.invokeAndWait(runner);
			return password[0];
		}
		catch (InterruptedException e) {
			logger.error("Unexpected exception while waiting for password", e);
			throw new ServerResponseException(I18n.tr("Abort by user."));
		}
		catch (InvocationTargetException e) {
			logger.error("Unexpected exception while waiting for password", e);
			throw new ServerResponseException(I18n.tr("Abort by user."));
		}
	}
	
	public File getSettingsFile() {
		return new File(FileHelper.getHomeDir("jhylafax"), "jhlafax.prefs");
	}
	
	public <T> T runJob(final Job<T> job) throws Exception {
		final Object[] retSuccess = new Object[1];
		final Exception[] ret = new Exception[1];
		final ProgressDialog dialog = new ProgressDialog("HylaFAX Server");
		
		Runnable runner = new Runnable() {
			public void run()
			{
				try {
					retSuccess[0] = job.run(dialog);
				}
				catch (Exception e) {
					ret[0] = e;
				}
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						dialog.done();
					}
				});
			}
		};

		Thread t = new Thread(runner, "Job");
		t.start();
		
		// show blocking dialog
		dialog.setLocationRelativeTo(this);
		dialog.showDialog();
		
		// at this point t is guranteed to be finished
		if (ret[0] != null) {
			throw ret[0];
		}
		return (T)retSuccess[0];
	}

	public void addLocaleChangeListener(LocaleChangeListener l) {
		localeListeners.add(l);
	}

	public void addTemporaryLocaleChangeListener(final JDialog dialog) {
		localeListeners.add((LocaleChangeListener)dialog);
		dialog.addWindowListener(new WindowAdapter() {
			public void windowClosed(WindowEvent e) {
				System.out.println("closed!");
				localeListeners.remove((LocaleChangeListener)dialog);
				dialog.removeWindowListener(this);
			}
		});
	}
	
	public void removeLocaleChangeListener(LocaleChangeListener l) {
		localeListeners.remove(l);
	}

	public void updateServerInfo() {
		serverInfoLabel.setText(I18n.tr("{0}:{1}", 
				Settings.HOSTNAME.getValue(), 
				Settings.PORT.getValue()));
	}
	
	public void updateLabels() {
		fileMenu.setText(I18n.tr("File"));
		aboutMenu.setText(I18n.tr("About"));
		
		for (int i = 0; i < mainTabbedPane.getTabCount(); i++) {
			QueuePanel panel = (QueuePanel)mainTabbedPane.getComponent(i);
			String queueName = panel.getQueueName();
			if (queueName.equals("recvq")) { mainTabbedPane.setTitleAt(i, I18n.tr("Received")); }
			else if (queueName.equals("sendq")) { mainTabbedPane.setTitleAt(i, I18n.tr("Sending")); }
			else if (queueName.equals("pollq")) { mainTabbedPane.setTitleAt(i, I18n.tr("Polling")); }
			else if (queueName.equals("doneq")) { mainTabbedPane.setTitleAt(i, I18n.tr("Done")); }
			else if (queueName.equals("docq")) { mainTabbedPane.setTitleAt(i, I18n.tr("Documents")); }
		}

		sendAction.putValue(Action.NAME, I18n.tr("Send Fax..."));
		sendAction.putValue(Action.SHORT_DESCRIPTION, I18n.tr("Opens a dialog for sending a fax"));

		pollAction.putValue(Action.NAME, I18n.tr("Poll Fax..."));
		pollAction.putValue(Action.SHORT_DESCRIPTION, I18n.tr("Opens a dialog for polling a fax"));

		updateStatusAction.putValue(Action.NAME, I18n.tr("Update Status"));
		updateStatusAction.putValue(Action.SHORT_DESCRIPTION, I18n.tr("Queries the status from the server"));
		
		settingsDialogAction.putValue(Action.NAME, I18n.tr("Settings..."));
		settingsDialogAction.putValue(Action.SHORT_DESCRIPTION, I18n.tr("Displays the settings dialog"));

		exitAction.putValue(Action.NAME, I18n.tr("Exit"));
		exitAction.putValue(Action.SHORT_DESCRIPTION, I18n.tr("Closes the application"));

		aboutAction.putValue(Action.NAME, I18n.tr("About..."));
		aboutAction.putValue(Action.SHORT_DESCRIPTION, I18n.tr("Opens a dialog that displays funny information"));
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		ThreadGroup tg = new ThreadGroup("JHylaFAXThreadGroup") {
			public void uncaughtException(Thread t,Throwable e) {
				e.printStackTrace();
			}
		};

//		System.setProperty("sun.awt.exception.handler", 
//						   "xnap.util.XNapAWTExceptionHandler");
		Thread mainRunner = new Thread(tg, "JHylaFAXMain") {
			public void run() {
				setContextClassLoader(JHylaFAX.class.getClassLoader());

				JHylaFAX app = new JHylaFAX();
				app.setVisible(true);
			}
		};
		mainRunner.start();		
	}

	public void updateTables()
	{
		Job<StatusResponse> ioJob = new Job<StatusResponse>() {
			@Override
			public StatusResponse run(ProgressMonitor monitor) throws Exception
			{				
				monitor.setTotalSteps(QUEUES.length + 2 /* connection + status */);
				monitor.setText(I18n.tr("Connecting..."));
				HylaFAXClient client = getConnection(monitor);
				monitor.work(1);
				
				StatusResponse response = new StatusResponse();
				
				monitor.setText(I18n.tr("Getting Status"));
				List<String> lines = client.getList("status");
				StringBuffer sb = new StringBuffer();
				for (Iterator<String> it = lines.iterator(); it.hasNext();) {
					String line = it.next();
					if (response.status == null) {
						response.status = line;
					}
					sb.append(line + "\n"); 
				}
				response.verboseStatus = sb.toString();
				monitor.work(1);
				
				client.jobfmt(HylaFAXClientHelper.JOBFMT);
				client.rcvfmt(HylaFAXClientHelper.RCVFMT);
				client.filefmt(HylaFAXClientHelper.FILEFMT);
				
				for (String name : QUEUES) {
					monitor.setText(I18n.tr("Getting {0}", name));
					List<HylaFAXJob> result = new ArrayList<HylaFAXJob>();
					lines = client.getList(name);
					for (String line : lines) {
						HylaFAXJob job = HylaFAXClientHelper.parseFmt(line);
						if (job != null) {
							result.add(job);
						}
					}
					response.queueByName.put(name, result);
					monitor.work(1);
				}
				
				return response;
			}				
		};
		
		try {
			StatusResponse response = JHylaFAX.getInstance().runJob(ioJob);
			updateServerInfo();
			statusLabel.setText(response.status);
			statusLabel.setToolTipText("<html><pre><b>" + response.verboseStatus + "</b></pre>");
			for (String queueName : response.queueByName.keySet()) {
				QueuePanel panel = queuePanelByName.get(queueName);
				if (panel != null) {
					panel.setData(response.queueByName.get(queueName));
				}
			}
		} 
		catch (UserAbortException e) {
		} 
		catch (Exception e) {
			logger.debug("Error updating status", e);
			ErrorDialog.showError(this, I18n.tr("Could not get status"), 
					I18n.tr("JHylaFAX Error"),
					e);
		}
	}

	private class AboutAction extends AbstractXNapAction {
		
		private AboutDialog dialog;
		
		public AboutAction() {
			putValue(ICON_FILENAME, "jhylafax.png");
		}

		public void actionPerformed(ActionEvent e)
		{
			if (dialog == null) {
				dialog = new AboutDialog();
				dialog.setTitle(I18n.tr("About JHylaFax"));
				dialog.setLocationRelativeTo(JHylaFAX.this);
				dialog.addLogoTab(I18n.tr("General Information"), IconHelper.getImage("jhylafax.png"),
						"README.txt");
				dialog.addTab(I18n.tr("License"), "LICENSE.txt"); 
			}
			
			if (!dialog.isVisible()) {
				dialog.setVisible(true);
			}
		}
	
	}

	private class SendAction extends AbstractXNapAction {
		
		public SendAction() {
			putValue(ICON_FILENAME, "mail_new.png");
		}

		public void actionPerformed(ActionEvent e)
		{
			SendDialog dialog = new SendDialog();
			addTemporaryLocaleChangeListener(dialog);
			dialog.setLocationRelativeTo(JHylaFAX.this);
			dialog.setVisible(true);
		}
	
	}

	private class PollAction extends AbstractXNapAction {
		
		public PollAction() {
			putValue(ICON_FILENAME, "mail_get.png");
		}

		public void actionPerformed(ActionEvent e)
		{
			PollDialog dialog = new PollDialog();
			addTemporaryLocaleChangeListener(dialog);
			dialog.setLocationRelativeTo(JHylaFAX.this);
			dialog.setVisible(true);
		}
	
	}

	private class SettingsDialogAction extends AbstractXNapAction {
		
		private SettingsDialog dialog;
		
		public SettingsDialogAction() {
			putValue(ICON_FILENAME, "configure.png");
		}

		public void actionPerformed(ActionEvent e)
		{
			if (dialog == null) {
				dialog = new SettingsDialog();
				dialog.setLocationRelativeTo(JHylaFAX.this);
				addLocaleChangeListener(dialog);
			}
			
			if (!dialog.isVisible()) {
				dialog.revert();
				dialog.setVisible(true);
			}
		}
		
	}

	private class ExitAction extends AbstractXNapAction {
		
		public ExitAction() {
			putValue(ICON_FILENAME, "exit.png");
		}

		public void actionPerformed(ActionEvent e)
		{
			exit();
		}
		
	}

	private class StatusResponse {
		String status;
		String verboseStatus;
		Map<String,List<HylaFAXJob>> queueByName = new HashMap<String, List<HylaFAXJob>>();
	}
	
	private class UpdateStatusAction extends AbstractXNapAction {
		
		public UpdateStatusAction() {
			putValue(ICON_FILENAME, "reload.png");
		}

		public void actionPerformed(ActionEvent e)
		{
			updateTables();
		}
		
	}

}
