/**
 * JHylaFax - A java client for HylaFAX.
 *
 * Copyright (C) 2005 by Steffen Pingel <steffenp@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package net.sf.jhylafax;

import gnu.hylafax.HylaFAXClient;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.swing.Action;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xnap.commons.gui.ColoredTable;
import org.xnap.commons.gui.Dialogs;
import org.xnap.commons.gui.ErrorDialog;
import org.xnap.commons.gui.action.AbstractXNapAction;
import org.xnap.commons.gui.table.TableLayout;
import org.xnap.commons.gui.table.TableSorter;
import org.xnap.commons.gui.util.GUIHelper;
import org.xnap.commons.i18n.I18n;
import org.xnap.commons.io.Job;
import org.xnap.commons.io.ProgressMonitor;
import org.xnap.commons.settings.SettingResource;
import org.xnap.commons.settings.TableSettingDirector;

/**
 * A generic panel that displays a list of {@link net.sf.jhylafax.HylaFAXJob} 
 * objects in a table.
 *  
 * @author Steffen Pingel
 */
public class QueuePanel extends JPanel implements ListSelectionListener, LocaleChangeListener {
	
	private final static Log logger = LogFactory.getLog(QueuePanel.class);
	
	private ViewAction viewAction;
	private SaveAction saveAction;
	private JobTableModel queueTableModel;
	private ColoredTable queueTable;
	private String queueName;
	private TableLayout queueTableLayout;

	private EditJobAction editAction;

	private RemoveJobAction removeAction;

	public QueuePanel(String queueName) {
		this.queueName = queueName;
		
		setLayout(new BorderLayout());
		setBorder(GUIHelper.createEmptyBorder(10));
		
		viewAction = new ViewAction();
		viewAction.setEnabled(false);
		saveAction = new SaveAction();
		saveAction.setEnabled(false);
		editAction = new EditJobAction();
		editAction.setEnabled(false);
		removeAction = new RemoveJobAction();
		removeAction.setEnabled(false);
		
		queueTableModel = new JobTableModel();
		TableSorter sorter = new TableSorter(queueTableModel);
		queueTable = new ColoredTable(sorter);
		queueTableLayout = new TableLayout(queueTable);
		queueTableLayout.setColumnProperties(0, "id", 20);
		queueTableLayout.setColumnProperties(1, "priority", 40);
		queueTableLayout.setColumnProperties(2, "result");
		queueTableLayout.setColumnProperties(3, "permissions");
		queueTableLayout.setColumnProperties(4, "owner");
		queueTableLayout.setColumnProperties(5, "resolution", 40);
		queueTableLayout.setColumnProperties(6, "number", 80);
		queueTableLayout.setColumnProperties(7, "time");
		queueTableLayout.setColumnProperties(8, "pages", 20);
		queueTableLayout.setColumnProperties(9, "dials", 20);
		queueTableLayout.setColumnProperties(10, "filename", 100);
		queueTableLayout.setColumnProperties(11, "size", 40);
		queueTableLayout.setColumnProperties(12, "error", 100);
		add(new JScrollPane(queueTable), BorderLayout.CENTER);

		queueTable.setShowVerticalLines(true);
		queueTable.setShowHorizontalLines(false);
		queueTable.setAutoCreateColumnsFromModel(true);
		queueTable.setIntercellSpacing(new java.awt.Dimension(2, 1));
		queueTable.setBounds(0, 0, 50, 50);
		// user code begin {1}
		queueTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		queueTable.getSelectionModel().addListSelectionListener(this);
		
		JPanel buttonPanel = new JPanel(new FlowLayout());
		add(buttonPanel, BorderLayout.SOUTH);
		// TODO: should not hard code "sendq" here
		if (queueName.equals("sendq")) {
			buttonPanel.add(editAction.createJButton());
			buttonPanel.add(removeAction.createJButton());
		}
		else {
			buttonPanel.add(viewAction.createJButton());
			buttonPanel.add(saveAction.createJButton());
		}
		updateLabels();
	}
	
	public void saveLayout(SettingResource backstore) {
		new TableSettingDirector(backstore, getQueueName()).save(queueTableLayout);
	}
	
	public void restoreLayout(SettingResource backstore, String[] defaultColumns) {
		new TableSettingDirector(backstore, getQueueName())
			.setDefaults(defaultColumns)
			.restore(queueTableLayout);
	}
	
	public void updateLabels() {
		viewAction.putValue(Action.NAME, "View");
		viewAction.putValue(Action.SHORT_DESCRIPTION, "");
	
		saveAction.putValue(Action.NAME, "Save");
		saveAction.putValue(Action.SHORT_DESCRIPTION, "");

		editAction.putValue(Action.NAME, "Edit Job...");
		editAction.putValue(Action.SHORT_DESCRIPTION, "");

		removeAction.putValue(Action.NAME, "Remove Job");
		removeAction.putValue(Action.SHORT_DESCRIPTION, "");

		queueTableLayout.setColumnNames(new String[] {
				I18n.tr("ID"),
				I18n.tr("Priority"), 
				I18n.tr("Result"),
				I18n.tr("Permission"),
				I18n.tr("Owner"),
				I18n.tr("Resolution"),
				I18n.tr("Number"),
				I18n.tr("Time"),
				I18n.tr("Pages"),
				I18n.tr("Dials"),				
				I18n.tr("Filename"), 
				I18n.tr("Size"), 
				I18n.tr("Last Error") }); 
	}
	
	private class ViewAction extends AbstractXNapAction {
		
		public ViewAction() {
			//FIXME putValue(ICON_FILENAME, "reload.png");
		}

		public void actionPerformed(ActionEvent event)
		{
			String viewerPath = Settings.VIEWER_PATH.getValue().trim();
			if (viewerPath.length() == 0) {
				Dialogs.showError(JHylaFAX.getInstance(), 
						GUIHelper.tt(I18n.tr("Could not open file: Please enter the path of an external viewer in the settings.")), 
						I18n.tr("JHylaFAX Error"));
				return;
			}
			
			HylaFAXJob job = getSelectedJob();
			File tempFile;
			try {
				tempFile = File.createTempFile("jhylafax", null);
			}
			catch (IOException e) {
				logger.debug("Error creating temporary file", e);
				ErrorDialog.showError(JHylaFAX.getInstance(), 
						I18n.tr("Error creating temporary file"),
						I18n.tr("JHylaFAX Error"),
						e);
				return;				
			}
			if (save(job.getFilename(), tempFile)) {
				try {
					Runtime.getRuntime().exec(new String[] {
							viewerPath, tempFile.getAbsolutePath()
					});
				}
				catch (IOException e) {
					logger.debug("Error executing viewer " + viewerPath + " "+ tempFile.getAbsolutePath(), e);
					ErrorDialog.showError(JHylaFAX.getInstance(), 
							I18n.tr("Could not execute viewer"),
							I18n.tr("JHylaFAX Error"),
							e);					
				}
				tempFile.deleteOnExit();
			}
		}
		
	}

	void removeJob(final int jobID) {
		Job<?> ioJob = new Job() {
			@Override
			public Object run(ProgressMonitor monitor) throws Exception
			{
				monitor.setTotalSteps(2);
				
				HylaFAXClient client = JHylaFAX.getInstance().getConnection(monitor);
				monitor.work(1);
				
				gnu.hylafax.Job editJob = client.getJob(jobID);
				client.kill(editJob);
				monitor.work(1);

				return null;
			}
		};
		
		try {
			JHylaFAX.getInstance().runJob(ioJob);
			JHylaFAX.getInstance().updateTables();
		} 
		catch (UserAbortException e) {
		} 
		catch (Exception e) {
			logger.debug("Error killing job", e);
			ErrorDialog.showError(JHylaFAX.getInstance(), 
					I18n.tr("Could not remove job"), 
					I18n.tr("JHylaFAX Error"),
					e);
		}
	}
	
	boolean save(final String filename, final File file) {
		final String absFilename = getQueueName() + "/" + filename;
		Job<?> ioJob = new Job() {
			@Override
			public Object run(ProgressMonitor monitor) throws Exception
			{
				monitor.setTotalSteps(3);
				
				HylaFAXClient client = JHylaFAX.getInstance().getConnection(monitor);
				monitor.work(1);
				
				client.mode(HylaFAXClient.MODE_STREAM);
				client.type(HylaFAXClient.TYPE_IMAGE);
				
				OutputStream out = new BufferedOutputStream(new FileOutputStream(file));
				try {
					client.get(absFilename, out);
				}
				finally {
					out.close();
				}
				monitor.work(2);
				return null;
			}
		};
		
		try {
			JHylaFAX.getInstance().runJob(ioJob);
		} 
		catch (UserAbortException e) {
			return false;
		}
		catch (Exception e) {
			logger.debug("Error getting file " + absFilename, e);
			ErrorDialog.showError(this, I18n.tr("Could not download file"), 
					I18n.tr("JHylaFAX Error"), e);
			return false;
		}
		return true;
	}
	
	private class SaveAction extends AbstractXNapAction {
		
		public SaveAction() {
			//FIXME putValue(ICON_FILENAME, "reload.png");
		}

		public void actionPerformed(ActionEvent e)
		{
			HylaFAXJob job = getSelectedJob();
			JFileChooser chooser = new JFileChooser();
			chooser.setSelectedFile(new File(job.getFilename()));
			if (chooser.showSaveDialog(JHylaFAX.getInstance()) == JFileChooser.APPROVE_OPTION) {
				// TODO show warning, if file exists
				save(job.getFilename(), chooser.getSelectedFile());
			}
		}
		
	}

	private class EditJobAction extends AbstractXNapAction {
		
		public EditJobAction() {
			//FIXME putValue(ICON_FILENAME, "reload.png");
		}

		public void actionPerformed(ActionEvent e)
		{
			HylaFAXJob job = getSelectedJob();
			EditDialog dialog = new EditDialog(job);
			dialog.setLocationRelativeTo(JHylaFAX.getInstance());
			dialog.setVisible(true);
		}
		
	}

	private class RemoveJobAction extends AbstractXNapAction {
		
		public RemoveJobAction() {
			//FIXME putValue(ICON_FILENAME, "reload.png");
		}

		public void actionPerformed(ActionEvent event)
		{
			HylaFAXJob job = getSelectedJob();
			if (Dialogs.showConfirmDialog(JHylaFAX.getInstance(), 
					I18n.tr("Do you really want to delete the job with id {0}?", "" + job.getID()),
					I18n.tr("Remove Job"), 
					JOptionPane.YES_NO_OPTION, 
					Settings.CONFIRM_DELETE) == JOptionPane.YES_OPTION) {
				removeJob(job.getID());
			}
		}
		
	}

	private static class JobTableModel extends AbstractTableModel {

		private List<HylaFAXJob> data = new ArrayList<HylaFAXJob>();
		
		private static final Class[] columnClasses= {
			Integer.class,
			Integer.class,
			String.class, 
			String.class,
			String.class,
			String.class,
			String.class,
			Date.class,
			Integer.class, 
			String.class,
			String.class,
			Long.class, 
			String.class,
		};
		
		public JobTableModel() 
		{
		}
		
		public void setData(List<HylaFAXJob> data)
		{
			this.data = data;
			fireTableDataChanged();
		}
	
		public Object getValueAt(int row, int column)
		{
			HylaFAXJob job = data.get(row);
			switch (column) {
			case 0:
				return job.getID();
			case 1:
				return job.getPriority();
			case 2:
				return job.getResult();
			case 3:
				return job.getPermissions();
			case 4:
				return job.getOwner();
			case 5:
				return job.getResolution();
			case 6:
				return job.getNumber();
			case 7:
				return job.getSendTime();
			case 8:
				return job.getPageCount();
			case 9:
				return job.getDialsAttempted() + "/" + job.getMaxDials();
			case 10:
				return job.getFilename();
			case 11:
				return job.getFilesize();
			case 12:
				return job.getLastError();
			default:
				return null;
			}
		}

		public int getRowCount()
		{
			return data.size();
		}

		public int getColumnCount()
		{
			return columnClasses.length;
		}
		
	    public Class<?> getColumnClass(int column) {
	        return columnClasses[column];
	    }
		
		public HylaFAXJob getJob(int row) {
			return data.get(row);
		}

	}

	public void setData(List<HylaFAXJob> date)
	{
		queueTableModel.setData(date);
	}


	public String getQueueName()
	{
		return queueName;
	}

	public HylaFAXJob getSelectedJob()
	{
		int i = queueTable.getSelectedRow();
		return (i != -1) ? queueTableModel.getJob(i) : null;
	}
	
	public void valueChanged(ListSelectionEvent e)
	{
		HylaFAXJob job = getSelectedJob();
		boolean isDownloadable = (job != null && job.getFilename() != null);
		viewAction.setEnabled(isDownloadable);
		saveAction.setEnabled(isDownloadable);
		boolean isEditable = (job != null) && job.getID() != -1;
		editAction.setEnabled(isEditable);
		removeAction.setEnabled(isEditable);
	}
	
}
