/**
 * JHylaFax - A java client for HylaFAX.
 *
 * Copyright (C) 2005 by Steffen Pingel <steffenp@gmx.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package net.sf.jhylafax;

import java.awt.Color;
import java.util.Locale;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.xnap.commons.gui.DefaultDialog;
import org.xnap.commons.gui.FileChooserPanel;
import org.xnap.commons.gui.ValidatedTextField;
import org.xnap.commons.i18n.I18n;
import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

/**
 * A dialog for editing of global settings.
 * 
 * @author Steffen Pingel
 */
public class SettingsDialog extends DefaultDialog implements LocaleChangeListener {

	private static final int DEFAULT_COLUMNS = 20;
	
	private JPanel generalPanel;
	private JLabel hostnameLabel;
	private JTextField hostnameTextField;
	private JPanel jobPanel;
	private JobPanel jobParameterPanel;
	private JTabbedPane mainTabbedPane;
	private JLabel passwordLabel;
	private JTextField passwordTextField;
	private JLabel portLabel;
	private ValidatedTextField portTextField;
	private JLabel usernameLabel;
	private JTextField usernameTextField;
	private JLabel languageLabel;
	private DefaultComboBoxModel languageModel;
	private JCheckBox confirmNonPSCheckBox;
	private JCheckBox confirmDeleteCheckBox;
	private JRadioButton internalCoverRadionButton;
	private JLabel internalCoverLabel;
	private JLabel viewerPathLabel;
	private JPanel pathPanel;
	private JRadioButton externalCoverRadionButton;
	private FileChooserPanel externalCoverPathFileChooserPanel;
	private FileChooserPanel viewerPathFileChooserPanel;

	private JLabel passwordInfoLabel;
	
	public SettingsDialog() {
		super(BUTTON_OKAY | BUTTON_APPLY | BUTTON_CANCEL);
		
		mainTabbedPane = new JTabbedPane();
		setMainComponent(mainTabbedPane);
		
		initializeGeneralForm();
		initializeJobForm();
		initializePathForm();
		
		updateLabels();
		revert();
		
		pack();
	}
	
	@Override
	public boolean apply() {
		Settings.HOSTNAME.setValue(hostnameTextField.getText());
		Settings.PORT.setValue(portTextField.getIntValue());
		Settings.USERNAME.setValue(usernameTextField.getText());
		Settings.PASSWORD.setValue(passwordTextField.getText());
		Settings.LOCALE.setValue((Locale)languageModel.getSelectedItem());
		Settings.CONFIRM_NONPS.setValue(confirmNonPSCheckBox.isSelected());
		Settings.CONFIRM_DELETE.setValue(confirmDeleteCheckBox.isSelected());
		
		jobParameterPanel.applyToSettings();
		
		Settings.VIEWER_PATH.setValue(viewerPathFileChooserPanel.getFilename());
		Settings.USE_INTERNAL_COVER.setValue(internalCoverRadionButton.isSelected());
		Settings.COVER_PATH.setValue(externalCoverPathFileChooserPanel.getFilename());
		return true;
	}

	private DefaultFormBuilder createForm() {
		FormLayout layout = new FormLayout("right:max(40dlu;pref), 3dlu, pref, pref:grow(0.5), pref:grow(0.5)", "");
		DefaultFormBuilder builder = new DefaultFormBuilder(layout);
		builder.setDefaultDialogBorder();
		return builder;
	}

	public void initializeGeneralForm() {
		DefaultFormBuilder builder = createForm();
		generalPanel = builder.getPanel();
		mainTabbedPane.add(generalPanel);
		
		builder.appendSeparator(I18n.tr("General"));
		
		hostnameTextField = new JTextField(DEFAULT_COLUMNS);
		hostnameLabel = builder.append("", hostnameTextField, 3);
		builder.nextLine();
		
		portTextField = new ValidatedTextField(5, ValidatedTextField.NUMBERS_INT);
		portLabel = builder.append("", portTextField);
		builder.nextLine();
		
		builder.appendSeparator(I18n.tr("Authentification"));
		
		usernameTextField = new JTextField(DEFAULT_COLUMNS);
		usernameLabel = builder.append("", usernameTextField, 3);
		builder.nextLine();
		
		passwordTextField = new JPasswordField(DEFAULT_COLUMNS);
		passwordLabel = builder.append("", passwordTextField, 3);
		builder.nextLine();
		
		passwordInfoLabel = new JLabel();
		passwordInfoLabel.setForeground(Color.RED);
		builder.append(passwordInfoLabel, 5);
		builder.nextLine();
		
		builder.appendSeparator(I18n.tr("General"));
		
		languageModel = new DefaultComboBoxModel();
		JComboBox lanuageComboBox = new JComboBox(languageModel);
		lanuageComboBox.setRenderer(new DefaultListCellRenderer() {
			public java.awt.Component getListCellRendererComponent(javax.swing.JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				if (value != null) {
					setText(((Locale)value).getDisplayName());
				}
				return this;
			}
		});
		languageLabel = builder.append("", lanuageComboBox, 2);
		builder.nextLine();
		
		confirmNonPSCheckBox = new JCheckBox();
		builder.append("", confirmNonPSCheckBox, 3);
		builder.nextLine();
		
		confirmDeleteCheckBox = new JCheckBox();
		builder.append("", confirmDeleteCheckBox, 3);
		builder.nextLine();
	}
	
	private void initializeJobForm() {
		DefaultFormBuilder builder = createForm();
		jobPanel = builder.getPanel();
		mainTabbedPane.add(jobPanel);
		
		jobParameterPanel = new JobPanel();
		builder.append(jobParameterPanel, 5);
	}
	
	public void initializePathForm() {
		DefaultFormBuilder builder = createForm();
		pathPanel = builder.getPanel();
		mainTabbedPane.add(pathPanel);
		
		builder.appendSeparator(I18n.tr("Programs"));
		
		viewerPathFileChooserPanel = new FileChooserPanel(DEFAULT_COLUMNS);
		viewerPathFileChooserPanel.setDialogParent(this);
		viewerPathLabel = builder.append("", viewerPathFileChooserPanel, 3);
		builder.nextLine();
		
		builder.appendSeparator(I18n.tr("Cover"));
		
		internalCoverRadionButton = new JRadioButton();
		internalCoverRadionButton.setSelected(true);
		internalCoverLabel = builder.append("", internalCoverRadionButton);
		builder.nextLine();
		
		externalCoverRadionButton = new JRadioButton();
		externalCoverPathFileChooserPanel = new FileChooserPanel(DEFAULT_COLUMNS);
		externalCoverPathFileChooserPanel.setEnabled(false);
		externalCoverPathFileChooserPanel.setDialogParent(this);
		externalCoverRadionButton.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				externalCoverPathFileChooserPanel.setEnabled(externalCoverRadionButton.isSelected());
			}			
		});
		builder.append("", externalCoverRadionButton);
		builder.nextLine();
		
		builder.append("", externalCoverPathFileChooserPanel, 3);
		builder.nextLine();
		
		ButtonGroup buttonGroup = new ButtonGroup();
		buttonGroup.add(internalCoverRadionButton);
		buttonGroup.add(externalCoverRadionButton);
	}
	
	
	public void defaults() {
		hostnameTextField.setText(Settings.HOSTNAME.getDefaultValue());
		portTextField.setText("" + Settings.PORT.getDefaultValue());
		usernameTextField.setText(Settings.USERNAME.getDefaultValue());
		passwordTextField.setText(Settings.PASSWORD.getDefaultValue());
		languageModel.setSelectedItem(Settings.LOCALE.getDefaultValue());
		confirmNonPSCheckBox.setSelected(Settings.CONFIRM_NONPS.getDefaultValue());
		confirmDeleteCheckBox.setSelected(Settings.CONFIRM_DELETE.getDefaultValue());
		
		jobParameterPanel.revertFromDefaultSettings();		
		
		viewerPathFileChooserPanel.setFilename(Settings.VIEWER_PATH.getDefaultValue());
		internalCoverRadionButton.setSelected(Settings.USE_INTERNAL_COVER.getDefaultValue());
		externalCoverPathFileChooserPanel.setFilename(Settings.COVER_PATH.getDefaultValue());
	}
	
	//FIXME @Override
	public void revert() {
		hostnameTextField.setText(Settings.HOSTNAME.getValue());
		portTextField.setText("" + Settings.PORT.getValue());
		usernameTextField.setText(Settings.USERNAME.getValue());
		passwordTextField.setText(Settings.PASSWORD.getValue());
		languageModel.setSelectedItem(Settings.LOCALE.getValue());
		confirmNonPSCheckBox.setSelected(Settings.CONFIRM_NONPS.getValue());
		confirmDeleteCheckBox.setSelected(Settings.CONFIRM_DELETE.getValue());
		
		jobParameterPanel.revertFromSettings();
		
		viewerPathFileChooserPanel.setFilename(Settings.VIEWER_PATH.getValue());
		internalCoverRadionButton.setSelected(Settings.USE_INTERNAL_COVER.getValue());
		externalCoverPathFileChooserPanel.setFilename(Settings.COVER_PATH.getValue());
	}
	
	public void updateLabels() {
		setTitle(I18n.tr("Settings"));
		
		mainTabbedPane.setTitleAt(mainTabbedPane.indexOfComponent(generalPanel), I18n.tr("Connection"));	
		hostnameLabel.setText(I18n.tr("Host"));
		portLabel.setText(I18n.tr("Port"));
		usernameLabel.setText(I18n.tr("Username"));
		passwordLabel.setText(I18n.tr("Password"));
		passwordInfoLabel.setText("Warning: The password is saved in plain text!");
		Object selectedItem = languageModel.getSelectedItem();
		languageModel.removeAllElements();
		for (Locale locale : JHylaFAX.SUPPORTED_LOCALES) {
			languageModel.addElement(locale);
		}
		languageModel.setSelectedItem(selectedItem);
		languageLabel.setText(I18n.tr("Lanuage"));
		confirmNonPSCheckBox.setText(I18n.tr("Confirm Sending Non-Postscript Documents"));
		confirmDeleteCheckBox.setText(I18n.tr("Confirm Delete"));

		mainTabbedPane.setTitleAt(mainTabbedPane.indexOfComponent(jobPanel), I18n.tr("Fax"));
		jobParameterPanel.updateLabels();
		
		mainTabbedPane.setTitleAt(mainTabbedPane.indexOfComponent(pathPanel), I18n.tr("Paths"));
		viewerPathLabel.setText(I18n.tr("Fax Viewer"));
		internalCoverLabel.setText(I18n.tr("Filename"));
		internalCoverRadionButton.setText(I18n.tr("Use Internal Cover"));
		externalCoverRadionButton.setText(I18n.tr("Use Custom PostScript File"));
	}
	
}
